package command;

public class TicTacToeBoard implements Memento<String> {
	private Cell[][] board;

	public TicTacToeBoard() {
		this.board = new Cell[3][3];
		clear();
	}

	public boolean placeX(int x, int y) {
		if (x < 0 || y < 0 || x >= 3 || y >= 3 || board[y][x] != Cell.EMPTY) {
			return false;
		}
		board[y][x] = Cell.X;
		return true;
	}

	public boolean placeO(int x, int y) {
		if (x < 0 || y < 0 || x >= 3 || y >= 3 || board[y][x] != Cell.EMPTY) {
			return false;
		}
		board[y][x] = Cell.O;
		return true;
	}

	public void clear() {
		for (int i = 0; i < 3; ++i) {
			for (int j = 0; j < 3; ++j) {
				board[i][j] = Cell.EMPTY;
			}
		}
	}

	public void print() {
		for (int i = 0; i < 3; ++i) {
			if (i != 0) {
				System.out.println("---+---+---");
			}
			for (int j = 0; j < 3; ++j) {
				if (j != 0) {
					System.out.print(" | ");
				} else {
					System.out.print(' ');
				}
				switch (board[i][j]) {
				case EMPTY:
					System.out.print(' ');
					break;
				case X:
					System.out.print('X');
					break;
				case O:
					System.out.print('O');
					break;
				}
			}
			System.out.println(' ');
		}
		System.out.println();
	}
	
	@Override
	public String save() {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < 3; ++i)
			for (int j = 0; j < 3; ++j)
				switch (board[i][j]) {
				case X:
					sb.append('X');
					break;
				case O:
					sb.append('O');
					break;
				case EMPTY:
					sb.append(' ');
					break;
				}
		return sb.toString();
	}

	@Override
	public void load(String s) {
		assert (s.length() == 9);
		s.chars().forEachOrdered(c -> {
			assert (c == 'X' || c == 'O' || c == ' ');
		});
		int c = 0;
		for (int i = 0; i < 3; ++i) {
			for (int j = 0; j < 3; ++j) {
				switch (s.charAt(c)) {
				case 'X':
					board[i][j] = Cell.X;
					break;
				case 'O':
					board[i][j] = Cell.O;
					break;
				case ' ':
					board[i][j] = Cell.EMPTY;
					break;
				}
				++c;
			}
		}
	}
}
