package command;

public abstract class UndoableCommand<SaveType> implements Command {

	private boolean run;
	private Memento<SaveType> memento;
	private SaveType state;

	public UndoableCommand(Memento<SaveType> m) {
		this.run = false;
		this.memento = m;
	}

	protected abstract void onExecute();

	public void execute() {
		if (!run) {
			state = memento.save();
			onExecute();
			run = true;
		}
	}

	public void undo() {
		if (run) {
			memento.load(state);
			run = false;
		}
	}
}
