package observer;

import java.util.ArrayList;
import java.util.List;

public class Observable<E> {

	private List<Observer<E>> mObservers;
	private E mData;
	
	public Observable() {
		this(null);
	}
	
	public Observable(E data) {
		this.mObservers = new ArrayList<>();
		this.mData = data;
	}
	
	public void attach(Observer<E> observer) {
		mObservers.add(observer);
	}
	
	public void detach(Observer<E> observer) {
		mObservers.remove(observer);
	}

	public void notifyAllObservers() {
		mObservers.forEach(o -> o.onUpdate(mData));
	}
	
	public void set(E data) {
		this.mData = data;
		notifyAllObservers();
	}
	
	public E get() {
		return this.mData;
	}
}
