package strategy;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class Main {

	static final Map<String, SortStrategy> algorithms =
			Map.of("default", new DefaultSortStrategy(),
					"merge", new MergeSortStrategy(),
					"selection", new SelectionSortStrategy());

	public static <E extends Comparable<E>> E[] sortByName(String name, E[] array) {
		E[] copy = array.clone();
		algorithms.get(name).sort(copy);
		return copy;
	}

	public static void main(String[] args) {
		Integer[] array = { 18, 15, 6, 11, 2, 16, 9, 19, 1, 10, 3, 14, 20, 8, 4, 7, 17, 5, 12, 0, 13 };
		var defaultSort = sortByName("default", array);
		var mergeSort = sortByName("merge", array);
		var selectionSort = sortByName("selection", array);

		System.out.println(Arrays.toString(array));
		System.out.println(Arrays.toString(defaultSort));
		System.out.println(Arrays.toString(mergeSort));
		System.out.println(Arrays.toString(selectionSort));
	}

}
