package visitor;

public class EvaluationVisitor implements ExpressionVisitor<Double> {

	private double[] vars;
	
	public EvaluationVisitor(double... vars) {
		this.vars = vars;
	}
	
	public double evaluate(Expression e) {
		return e.accept(this);
	}
	
	public double evaluate(Expression e, double... vars) {
		this.vars = vars;
		return e.accept(this);
	}
	
	@Override
	public Double visit(AddExpression e) {
		var lhs = e.getLhs().accept(this);
		var rhs = e.getRhs().accept(this);
		return lhs + rhs;
	}

	@Override
	public Double visit(SubExpression e) {
		var lhs = e.getLhs().accept(this);
		var rhs = e.getRhs().accept(this);
		return lhs - rhs;
	}

	@Override
	public Double visit(MulExpression e) {
		var lhs = e.getLhs().accept(this);
		var rhs = e.getRhs().accept(this);
		return lhs * rhs;
	}

	@Override
	public Double visit(DivExpression e) {
		var lhs = e.getLhs().accept(this);
		var rhs = e.getRhs().accept(this);
		return lhs / rhs;
	}

	@Override
	public Double visit(LogExpression e) {
		var num = e.getNum().accept(this);
		var base = e.getBase().accept(this);
		return Math.log(num) / Math.log(base);
	}

	@Override
	public Double visit(PowerExpression e) {
		var base = e.getBase().accept(this);
		var exponent = e.getExponent().accept(this);
		return Math.pow(base, exponent);
	}

	@Override
	public Double visit(NegateExpression e) {
		return -e.get().accept(this);
	}

	@Override
	public Double visit(Variable v) {
		return vars[v.getIndex()];
	}

	@Override
	public Double visit(Constant c) {
		return c.getValue();
	}

}
