package visitor;

public class LispPrintVisitor implements ExpressionVisitor<StringBuilder> {

	private StringBuilder sb;
	
	public LispPrintVisitor() {
		sb = new StringBuilder();
	}
	
	public void print(Expression e) {
		sb.setLength(0);
		e.accept(this);
		System.out.println(sb.toString());
	}

	@Override
	public StringBuilder visit(AddExpression e) {
		sb.append('(').append('+').append(' ');
		e.getLhs().accept(this);
		sb.append(' ');
		e.getRhs().accept(this);
		sb.append(')');
		return sb;
	}

	@Override
	public StringBuilder visit(SubExpression e) {
		sb.append('(').append('-').append(' ');
		e.getLhs().accept(this);
		sb.append(' ');
		e.getRhs().accept(this);
		sb.append(')');
		return sb;
	}

	@Override
	public StringBuilder visit(MulExpression e) {
		sb.append('(').append('*').append(' ');
		e.getLhs().accept(this);
		sb.append(' ');
		e.getRhs().accept(this);
		sb.append(')');
		return sb;
	}

	@Override
	public StringBuilder visit(DivExpression e) {
		sb.append('(').append('/').append(' ');
		e.getLhs().accept(this);
		sb.append(' ');
		e.getRhs().accept(this);
		sb.append(')');
		return sb;
	}

	@Override
	public StringBuilder visit(LogExpression e) {
		sb.append('(').append("log").append(' ');
		e.getBase().accept(this);
		sb.append(' ');
		e.getNum().accept(this);
		sb.append(')');
		return sb;
	}

	@Override
	public StringBuilder visit(PowerExpression e) {
		sb.append('(').append('^').append(' ');
		e.getBase().accept(this);
		sb.append(' ');
		e.getExponent().accept(this);
		sb.append(')');
		return sb;
	}

	@Override
	public StringBuilder visit(NegateExpression e) {
		sb.append('(').append('-').append(' ');
		e.get().accept(this);
		sb.append(')');
		return sb;
	}

	@Override
	public StringBuilder visit(Variable v) {
		sb.append(v.toString());
		return sb;
	}

	@Override
	public StringBuilder visit(Constant c) {
		sb.append(Double.toString(c.getValue()));
		return sb;
	}
	

}
